#! /usr/bin/perl

use 5.008;
use File::Copy;
use IO::File;
use strict;

require "rcliper_config.pl";

require $Config::generalCode;
require $Config::rcliperDataCode;
require $Config::rcliperPlotCode;
require $Config::atcfStormsCode;
require $Config::btkDataCode;

#=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Program Name: rcliper.pl
#
# Version: 7.0
#
# Language: Perl 5.8.x
#
# Purpose: 	This script controls the creation of products related to the rcliper algorithm
#		which was written my Mark DeMaria (OAR) and Frank Marks (HRD).
#
#		There are several different products this program can create including:
#
#			-> Storm's total predicted accumulated rainfall over the 
#			   storm's lifetime
#				-> Postscript file
#				-> RCLIPER output text file
#	
#			-> Storm's forecasted total accumulated rainfall over a user selected
#			   time period for a user selected date/time group.  If a time period is 
#			   entered by the user then this algorithm will be run for all storms 
#			   active at the given time as determined by the Best track file. 
#
#			   This program can calculate the total rainfall over the next
#			   12, 24, 36, 48 or 72 hour period. 
#
#				-> Postscript file
#				-> RCLIPER output text file
#				-> GEMPAK grid file 
#				-> Storm GEMPAK grid file viewable in NMAP2
#				   which contains all of the separate grid files
#				   produced for that specific storm.
#
#		Defaults -
#
#		If the user does not select a specific date/time group the default
#		will be the current date/time when the algorithm is run.  If the user
#		does not select a specific forecast time the default of 72 hours will be selected.
#
#
#		The user can select to create different products from the command line
#		using the syntax shown below in the Usage section.
#
# Usage:	rcliper.pl -plot total -storm storm_name 
#		rcliper.pl -plot period -date YYYYMMDDHH -forecast (12|24|36|48|72)  
#		rcliper.pl -plot period -forecast (12|24|36|48|72)
#
# Example:	-> Plot the storm total rainfall for al032003
#		
#	 		rcliper.pl -plot total -storm al032003
#              
#		- Plot the 24 hour forecasted rainfall for all active storms at 20030908 at 0z
#	      
#	     		rcliper.pl -plot period -date 2003090800 -forecast 24
#              
#		- Plot the 36 hour forecasted rainfall for all of the active storms at the current time
#		
#	      		rcliper.pl -plot period -forecast 36
#			
#		- Plot the 72 hour forecasted rainfall for the current time	
#	     
#	      		rcliper.pl -plot period
#
#------------------------------------------------------------
# Calls: 
#		rcliper_algorithm.exe			->RCLIPER executable for a DTG
#
#			Can be either one of two versions of the rcliper code: TRMM data 
#			or the rain gauge data.
#
#			The rcliper executable (rcliper_algorithm.exe) must be copied
#			into the same directory as the computes file (comab.dat) and 
#			geographic coast line coordinates (aland.dat).  It can not be 
#			run in a separate directory.  The RCLIPER result text file
#			will be put in the same directory where rcliper.exe was run.
# 			See below for example of a computes file. 
#
#		rcliper_newi.exe		->RCLIPER executable for a complete storm
#
#			The rcliper executable (rcliper_newi.exe) must be copied
#			into the same directory as the computes file (comab.dat) and 
#			geographic coast line coordinates (aland.dat).  It can not be 
#			run in a separate directory.  The RCLIPER result text file
#			will be put in the same directory where rcliper.exe was run.
# 			See below for example of a computes file. 
#
#		rcliper_data_v3_1.pl
#
#			This perl script uses the text result file produced by the RCLIPER
#			algorithm and creates a data file in a format that can be used
#			with GEMPAK to create/plot the RCLIPER results.
#
#		rcliper_plot_v3_1.pl 
#
#			This perl script uses the data file produced by the rliper_data.pl script
#			and GEMPAK to create GEMPAK files/plots.
#
#		atcfStorms_v4.pl 
#
#			Provides a variety of subroutines to
#
#           		1. Identify the tropical cyclones active during a date/time
#           		2. Create a computes file for an active tropical cyclone
#           		3. Find the start/end dates of a tropical cyclone
#
#		generalUtils.pl
#
#			general file/date/time utilities
#
#		btk_data.pl
#
#			 Reads the best track file for an ATCF storm and extracts information 
#			 which is stored in an btk_data.pl data object.  This data can be 
#			 access through function calls.
#
# Called by: Cronjob or Command line
#
#--------------------------------------------------------
# Variables:
#
##--- Data files
#
#	$coastlineFile 		File containing US Coast Line points for RCLIPER algorithm
#	$computes		File containing computes file (example above)
#   	$rcliperInput 		File containing storm summary file
#	$cronProfile 		File containing GEMPAK environment variables 
#
##--- Executables
#
#	$rcliperExe 		RCLIPER Algorithm (produces a text file of lat/lon/rain)
#	$rcliperTotalExe 	RCLIPER Algorithm to produce a storm total rainfall postscript file
#	$asctogem		GEMPAK routine to convert ascii file to GEMPAK grids
#
##--- Random variables 
#
#	$lockFile 		Name of the lock file - Prevents more then one rcliper_v3.pl process at a time
#	$LOG 			File handle to the log file
#
##--- Log (s) file
#
#	$logFile = "rcliperProcess.log";	Log produced by this perl script
#	$rcliperLog = "rcliper.log";		Log produced by the RCLIPER algorithm ($rcliperExe)
#
#--------------------------------------------------------
# Examples of files:
#
#	Computes file: A compute file contains the OFCL and CARQ record for a storm.
#				   The OFCL points are 6 hours before the CARQ points.
#
#	example:		AL, 09, 2002091200, 03, OFCL,   0, 263N,  866W,  30, 1001, TD,  34, AAA,    0,    0,    0,    0,    0,    0,   0,  40,   0,    ,   0, RJP,  35,   4,
#					AL, 09, 2002091200, 03, OFCL,   3, 265N,  865W,  30, 1001, TD,  34, NEQ,    0,    0,    0,    0,    0,    0,   0,  40,   0,    ,   0, RJP,  35,   4,
#					AL, 09, 2002091200, 03, OFCL,  12, 270N,  861W,  30, 1001, TD,  34, AAA,    0,    0,    0,    0,    0,    0,   0,  40,   0,    ,   0, RJP,  35,   4,
#					AL, 09, 2002091200, 03, OFCL,  24, 275N,  859W,  35, 1001, TD,  34, NEQ,   75,   75,   50,   50,    0,    0,   0,  45,   0,    ,   0, RJP,  20,   3,
#					AL, 09, 2002091200, 03, OFCL,  36, 282N,  856W,  40, 1001, TD,  34, NEQ,   75,   75,   50,   50,    0,    0,   0,  50,   0,    ,   0, RJP,  20,   4,
#					AL, 09, 2002091200, 03, OFCL,  48, 290N,  853W,  45, 1001, TD,  34, NEQ,   75,   75,   50,   50,    0,    0,   0,  55,   0,    ,   0, RJP,  20,   4,
#					AL, 09, 2002091200, 03, OFCL,  72, 310N,  845W,  30, 1001, TD,  34, AAA,    0,    0,    0,    0,    0,    0,   0,  40,   0,    ,   0, RJP,  20,   5,
#					AL, 09, 2002091206, 01, CARQ, -24, 257N,  871W,  20,    0, XX,  34, AAA,    0,    0,    0,    0,    0,    0,   0,   0,   0,   L,   0,   X,   0,   0, NINE      ,  ,
#					AL, 09, 2002091206, 01, CARQ, -18, 257N,  871W,  20,    0, XX,  34, AAA,    0,    0,    0,    0,    0,    0,   0,   0,   0,   L,   0,   X,   0,   0, NINE      ,  ,
#					AL, 09, 2002091206, 01, CARQ, -12, 260N,  870W,  30,    0, XX,  34, AAA,    0,    0,    0,    0,    0,    0,   0,   0,   0,   L,   0,   X,  37,   4, NINE      ,  ,
#					AL, 09, 2002091206, 01, CARQ,  -6, 263N,  866W,  30,    0, XX,  34, AAA,    0,    0,    0,    0,    0,    0,   0,   0,   0,   L,   0,   X,   0,   0, NINE      ,  ,
#					AL, 09, 2002091206, 01, CARQ,   0, 267N,  864W,  30, 1001, XX,  34, NEQ,    0,    0,    0,    0, 1008,  150,  20,   0,   0,   L,   0,   X,  35,   4, NINE      , D,
#					AL, 09, 2002091206, 01, CARQ,   0, 267N,  864W,  30, 1001, XX,  50, NEQ,    0,    0,    0,    0, 1008,  150,  20,   0,   0,   L,   0,   X,  35,   4, NINE      , D,
#
#	Rcliper Text File: line1: Contains the name/date/tau of the information used to create the rcliper file.  
#					   line2: Information about the resolution of the rcliper grid and the lat/lon coordinates of the 
#					   		  LL and UR extent of the data coverage area.
#					   line3-7: The predicted storm position for the next 72 hours in 12 hour increments
#					   line8-end: The lat/long coordinates of the point along with a prediction of the amount
#								  of rain in inches at that location.
#
#	example:		   AL092002 091300 NINE
# 					   121    0.250 -102.000  -72.000 121    0.250   21.000   51.000
#  					   -87.90  26.60   0.0
#  					   -87.55  27.73  12.0
#  					   -87.02  29.30  24.0
#  					   -86.42  30.85  36.0
#  					   -84.85  33.22  48.0
#  					   -82.06  34.93  60.0
#   				   -0.00   0.00  72.0
# 					   -102.000   21.000    0.000
# 					   -101.750   21.000    0.000
# 					   -101.500   21.000    0.000		...etc
#
#	Rcliper Log File:
#
#	example:	
#					RCLIPER forecast for AL132003 060318 LILI
#
# 					Track/Intensity
#  			 		 0.  13.2   66.0  60.
#  			 		12.  14.2   68.5  65.
# 			 		24.  15.3   70.2  70.
# 			 		36.  16.9   71.7  78.
#  			 		48.  18.5   73.3  78.
#  			 		60.  19.7   74.4  68.
#  			 		72.  20.8   75.0  66.
# 			 		84.   0.0    0.0   0.
# 			 		96.   0.0    0.0   0.
#		    		108.   0.0    0.0   0.
#					120.   0.0    0.0   0.
#
# 					Track forecast out to  72. hrs
#
# 					Divide track into 145 points 0.5 hrs apart
#
# 					Storm inland at t= 45.0 lon/lat:   -72.9   18.2  dtl=  -1.9
#
# 					Rainfall grid reference lon/lat:   -73.0   18.0
#
# 					Max rain= 6.7 at lon/lat:  -74.000   19.500
#
# 					Output file name: 03060318AL13_rcliper.txt
#
#
# 	Reformated Rcliper File for GEMPAK:	Line 1: Contains YYMMDD/HH00 0 RAIN
#
#										Line 2-n: Contains lat/lon/rain amount in inches 
#			  							The values are listed from the upper left
#			  							corner of the result grid to the lower right corner.
# 	example:	
#
#				031008/1200  0  RAIN
#				13.000 -122.000 0.000
#				13.000 -121.750 0.000
#				13.000 -121.500 0.000
#				13.000 -121.250 0.000
#				13.000 -121.000 0.000  	...etc
#
#----------------------------------------------------------------------
# Information:
#    Joint Hurricane Testbed/USWRP project
#    National Hurricane Center / Tropical Prediction Center
#    Department of Commerce - NOAA / NWS
#
# Category: JHT - RCLIPER 
#
# Written by: Alison Krautkramer 09/2002
# Modified: 07/2003 
#		Added flags to control program options.
#	
#		Added commands to ftp results to dogfish (anonymous ftp server)
#		so HPC could obtain the ps and grid files.
#
#		Found error: the files() array was not reinitialized at the 
#		beginning of each loop and therefore the wrong grids were being
#		added to the NMAP 2 files.
# Modified: 10/2003
#		Reorganized code
#		Add forecast hour command line variable
# Modified: 07/2004
#		Added code to check for the rcliper text files before creating 
#		the gempak grids/plots.  If the rcliper text file is found
#		in the expected location then it is assumed that the grid or 
#		plot has already been created and therefore won't be recreated.
# Modified: 08/2004
#		Added a variable which indicates whether any new information has
#		been processed by this run of the RCLIPER algorithm.  If no new data
#		had been processed then the log file is not copied to the storage directory.
#		Otherwise, the log file was being cloppered. 
# Modified: 10/2004
#		Adapted the perl scripts to add all RCLIPER grids to the same
#		grid regardless of the forecast time period (f12, f24, f36, f48, f72).
#			line 574 -> new call to rcliper_data
#			line 900 -> new patter to find nmap grid
# Modified: 11/19/2004
#		A new version of this program was created after a reorganizing and 
#		streamlining of the version 6.0 code was completed.  The current code
#		contains more small subroutines.  Also, the creation of a single grid
#		file has been completely divorced from the creation/addition of a single
#		grid file to a nmap composite grid.  A new configuaration variable has been
#		created to separatly store the single grid files that can now be created and
#		stored.
# Modified: 12/02/2004
#		Change rename() commands to copy() followed by unlink().  The rename()
#		command can cause a cross link device error when trying to copy files
#		to a mounted file system.
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-



#===================================
# **** Variable Declaration
#===================================


#--- Data files

my $coastlineFile = "aland.dat";
my $computes = "comab.dat";
my $rcliperInput = "btkTotal.txt";

#--- Executables

my $rcliperExe = "rcliper_algorithm.exe";
my $rcliperTotalExe = "rcliper_total.exe";

#--- Lock file

my $lockFile = "lock";

#--- Log file

my $logFile = "rcliperProcess.log";
my $LOG = new IO::File;


#=====================================
# **** Data Structure Definitions
#=====================================


my $atcfObj;
my $object = {

		"algorithm" 	=> "",
		"processLog" 	=> "",
		"date"	 	=> "",
		"type" 		=> "",
		"raStorms" 	=> [],
		"rcliperLog" 	=> "",
		"forecastHour" 	=> ""
				
	};


			 

			 
#==================================================================
#                       *** START MAIN ***
#==================================================================





#==========================================
# 	*** Get user input  ***
#==========================================

userInput ($object);


#==========================================
# 	*** Set up working environment  ***
#==========================================
	
#--- Create the work directory, check lock file, open log and copy rcliper files
#--- to the work directory

setup ($object, \$LOG);




#=========================================================
#
#	***  ***
#
#=========================================================



#--- Check if there are any storms in the storms array

if (@{$$object{raStorms}} == 0) {

	findStorms($object, \$atcfObj);

}


my $newData = 0; #Keeps track of whether any new information was processed

foreach my $storm (@{$$object{raStorms}}) {

		
	#==========================================
	#  *** Create a rcliper input file ***
	#=========================================
	
	
	if ($$object{type} eq "period") {
	
		$atcfObj->stormComputes($storm);
	
		if ($atcfObj->createStormComputes($storm, $Config::workDir, $computes) != 1) {
	
			printf ("Error creating computes file for $storm. \n");
			next;
	
		}
		
	}
	else{
	
		if (btkInput($storm, "$Config::atcfBtkDir", $rcliperInput, $LOG) == -1) {
		
			next;
		
		}
	
	}
		
		

	#==========================================
	#  *** Run the RCLIPER Algorithm ***
	#=========================================
	
	
	#--- Print header to LOG
	
	generalUtils::header ("Run the RCLIPER algorithm on $storm", $LOG);
	
	#--- Execute RCLIPER Algorithm
	
	system("cd $Config::workDir; $$object{algorithm}  $$object{forecastHour}");
	
	#--- Add the RCLIPER Log file 

	printf ($LOG "Contents of $$object{rcliperLog}....\n");
	
	generalUtils::appendFile ($Config::workDir, $$object{rcliperLog} , $LOG);
	


	#=========================================
	#  *** Reformat RCLIPER text output ***
	#=========================================
	
	
	#--- Create an rcliper data object
		
	my ($rcliper, $renamedText);
		
	if (rcliperData(\$rcliper, $object, \$renamedText, $storm, $LOG) != 1) {
		
		next;
		
	}
		
	$newData = 1;	#indicates new data found
		
		
		
	#=========================================
	#  *** Plot RCLIPER text data ***
	#=========================================
		
		
	#--- Create an rcliper plot object
	
	my $plot = rcliper_plot::new($rcliper, $$object{forecastHour});	
	
	
	#--- Create Postscript File
	
	makePS($rcliper, $plot, $LOG) if $Config::createPS;
	
	
	#--- Create GEMPAK grid File
	
	if (($Config::createGrid == 1 || $Config::addGridNMAP == 1) && $$object{type} eq "period") {
	
		makeGrid($rcliper, $plot, $LOG);
		
	}
	
		
	#======================================================
	#	****  Rename the RCLIPER output text file ***
	#======================================================	
		
		
	if ($Config::storeResults == 1) {
			
		rename ("$Config::workDir/" . $rcliper->getText(), "$Config::resultDir/$renamedText");
	
	}
	else {
		
		rename ("$Config::workDir/" . $rcliper->getText(), "$Config::workDir/$renamedText");
	
	}

	#--- Remove compute file / grid file
	
	unlink("$Config::workDir/$computes");
	unlink("$Config::workDir/$$object{rcliperLog}");
	unlink("$Config::workDir/" . $rcliper->getData());
	
		
}


#==========================================
#  *** Clean up ***
#=========================================	


close ($LOG);	
clean();	
	
#--- Move log to storage directory

if ($Config::storeResults == 1 && $newData == 1) {

	rename ("$Config::workDir/$$object{processLog}", "$Config::psResultDir/$$object{processLog}");

}	
	
	
	
	
#==================================================================
#           *** GET USER INPUT  ***
#==================================================================
	
	

#--------------------------------------------------------
# Subroutine: userInput
#
# Purpose: Examines the user input and determines if it is valid.
#		   The variables are assigned to the valid input and 
#		   are returned to the calling function.
#
# Called by : getInput
#
# Calls: checkFhour() - checks entered forecast hour for validity
#	usage() - prints a syntax message to the screen
#
# Modification:
#
#
#--------------------------------------------------------

sub userInput {

	my ($rObject) = @_;
	
	
	#--- Provides command syntax help

	if (!defined ($ARGV[0]) || $ARGV[0] eq "help" || $ARGV[0] eq "-help" || $ARGV[0] eq "usage" || $ARGV[0] eq "-u") {

		usage();
		exit;

	}

	if (defined($ARGV[0]) && ($ARGV[0] eq "plot" || $ARGV[0] eq "-plot")) {


		#------------------------------------
		#
		# 	Plot an entire storm track
		#
		#------------------------------------


		if (defined($ARGV[1]) && ($ARGV[1] eq "total") && 
			defined($ARGV[2]) && ($ARGV[2] eq "storm" || $ARGV[2] eq "-storm")
			&& defined($ARGV[3])) {
	
			$$rObject{type} = "total";
			$$rObject{algorithm} = $rcliperTotalExe;
			$$rObject{rcliperLog} = "rcliperTotal.log";
			$$rObject{processLog} = $logFile . "_$ARGV[3]";
			$$rObject{forecastHour} = "";
			$ARGV[3] =~ tr/a-z/A-Z/;
			push (@{$$rObject{raStorms}}, $ARGV[3]);
		
		}
	
	
		#--------------------------------------
		#
		# 	Plot a specific date/time group
		#
		#--------------------------------------
	
	
		elsif (defined($ARGV[1]) && ($ARGV[1] eq "period")) {
	
			$$rObject{algorithm} = $rcliperExe;
			$$rObject{rcliperLog} = "rcliper.log";
			$$rObject{type} = "period";
	
			if (defined($ARGV[2]) && ($ARGV[2] eq "date" || $ARGV[2] eq "-date") && defined($ARGV[3])) {
			
				#---- Check the date format
				
				$$rObject{date} = $ARGV[3];
			
				if (generalUtils::validDateFormat($$rObject{date}) != 1){
			
					printf ("Supplied date in incorrect format\n");
					usage();
					exit;
			
				}
	
			}
			else {
		
				#--- Set the date variable to todays date/time
		
				if (generalUtils::synopticTime("today", 0, \$$rObject{date}) != 1) {

					print ("Error finding the current synoptic time for today");
					exit;
			
				}
	
			}
		
			my $time = 0;
			if (defined($ARGV[4]) && ($ARGV[4] eq "forecast" || $ARGV[4] eq "-forecast") && defined($ARGV[5])) {
		
				$time = $ARGV[5];
				#checkFhour($ARGV[5], $rObject);
		
			}
			elsif (defined($ARGV[2]) && ($ARGV[2] eq "forecast" || $ARGV[2] eq "-forecast") && defined($ARGV[3])) {
		
				$time = $ARGV[3];
				#checkFhour($ARGV[3], $rObject);
			
			}
			
			checkFhour($time, $rObject);
		
		
			$$rObject{processLog} = $logFile . "_$$rObject{date}f$$rObject{forecastHour}";
			
		
		}
		
		else {
	
			usage();
			exit;
		}
		
		return;
	
	}

	usage();
	exit;

}


#--------------------------------------------------------
# Subroutine: checkFhour
#
# Purpose: Check the user entered forecast time for validity.
#	   Sets the up the object with the 
#	   correct forecast hour type infomation.  If the forecast 
#	   hour is either not entered or is incorrect then 72 hours will
#	   be chosen by default.
#
#		Valid Choices: 12, 24, 36, 48, 72 
#						 
#						   
# Input: $choice (string) - the user entered graph type
#	$rObject (ref object) - RCliper object
#
# Called by: userInput
#--------------------------------------------------------

sub checkFhour{

	my ($choice, $rObject) = @_;
	
	
	if ($choice == 48 || $choice == 36 || $choice == 24 || $choice == 12 || $choice == 72) { 
			
		$$rObject{forecastHour} = $choice;
		return;
			
	}
	
	print ("Incorrect forecast time or the forecast time was not entered.\nSetting to default of 72 hours.\n");	
	$$rObject{forecastHour} = 72;
				
}


#--------------------------------------------------------
# Subroutine: usage
#
# Purpose: Prints Syntax message to the screen and ends the program.
#
# Called by : userInput
#
# Modification: 
#
#--------------------------------------------------------

sub usage {

	printf ("Usage: rcliper.pl -plot total -storm storm_name \n");
	printf ("       rcliper.pl -plot period [-date YYYYMMDDHH] [-forecast 12|24|36|48|72] \n"); 

}





#==================================================================
#            *** START SUBROUTINES ***
#==================================================================





#--------------------------------------------------------
# Subroutine: rcliperData
#
# Purpose: Creates a rcliper data object and rename the 
#		text file
#						 
#						   
# Input: 
#
# Called by: main
#--------------------------------------------------------
sub rcliperData {

	my ($rData, $rObject, $rTextFile, $storm, $LOG) = @_;
	

	#--- Create an rcliper data object
	
	$$rData = rcliper_data::new($$rObject{rcliperLog}, $$rObject{forecastHour}, $LOG);
	
	if (!defined($$rData)) {
	
		printf ($LOG "Error (rcliper::rcliperData): Problem creating an rcliper_data object. \n");
		return -1;
	
	}

	#--- Create new rcliper text file name

        $$rTextFile = $$rData->getText();

        if ($$rObject{type} eq "period") {

		$$rTextFile = substr($$rTextFile, 8, 4) . "_20" . substr($$rTextFile, 0, 8) . "f$$object{forecastHour}" . substr($$rTextFile, 12, 20);
               	$$rTextFile =~ tr/[A-Z]/[a-z]/;

        }
        elsif ($$rObject{type} eq "total") {

                $$rTextFile = $storm . substr($$rTextFile, 12, 20);

       	}

	#--- Check if this text file already exists ... ie already processed

	if (-e "$Config::resultDir/$$rTextFile") {

		print ($LOG "Warning:  This storm/period has already been plotted.\n");
		print ($LOG "Not plotting image.\n");
		return -2;
		
	}

	
	if ($$rData->reformatText($LOG) != 1) {
		
		printf ($LOG "Error: Problem reformating data into ASCTOGEM format.\n");
		return -1;
	
	}
		
	
	return 1;	#Success

}
	
#--------------------------------------------------------
# Subroutine: findStorms
#
# Purpose: Creates an atcf object which can be used to find
#	   all of the storms active at the date specified 
#	   by the user.
#						 
#						   
# Input: $rATCFobject (ref object) - 
#	$rObject (ref object) - RCliper object
#
# Called by: main
#--------------------------------------------------------
sub findStorms {


	my ($rObject, $rATCFobject) = @_;

	#--- Create the atcfObj and set the storms array
		
	#$atcfObj = atcfStorms::new($Config::atcfDir, $$rObject{date});
	$$rATCFobject = atcfStorms::new($Config::atcfBtkDir, $Config::atcfComDir, $Config::atcfAidDir, $$rObject{date});

	if (!defined $$rATCFobject) {

		printf ("Error creating atcfStorms object");
		unlink("$Config::workDir/$$object{processLog}");
		clean();
		exit;
	
	}

	#--- Find all TCs active at $date

	$rObject->{raStorms} = $$rATCFobject->getActiveStorms();

	if (@{$rObject->{raStorms}} == 0) {

		printf ("No Current Tropical Cyclones in ATCF");
		unlink("$Config::workDir/$$object{processLog}");
		clean();
		exit;

	}	


}


#--------------------------------------------------------
# Subroutine: btkInput 
#
# Purpose: Create a storm total input file for the rcliper algorithm.
#--------------------------------------------------------
sub btkInput {


	#--------------------------
	#*** Define Variables
	#--------------------------
	
	
	my ($storm, $btkDir, $file, $LOG) = @_;
	my $returnCode;
	my $OUTPUT = new IO::File;
	
	#--------------------------
	#*** Create Btk object
	#--------------------------


	my $btkObj = btk_data::new($storm, $Config::workDir, $btkDir, $LOG);
	
	if (!defined ($btkObj)) {
	
		printf ($LOG "Error - Can't create btk_data object for $storm.\n");
		return -1;
	
	}
	
	
	#--------------------------
	#*** Get hash of lat/long/dtg/intensity values
	#--------------------------
	
	my %hBtkValues;
	my @aOrderedDTG;
	$returnCode = btk_data::getIntensity($btkObj, \%hBtkValues, \@aOrderedDTG);
	
	
	#--------------------------
	#*** Open an output file
	#--------------------------
	
	
	$returnCode = generalUtils::openFile ($Config::workDir, $file, "write", \$OUTPUT);
	
	if ($returnCode != 1) {
	
		printf ($LOG "Error ($returnCode): Failed opening $file\n");
		return $returnCode;
		
	}
	
	printf ($OUTPUT "1 \n");
	printf ($OUTPUT substr($storm, 0, 2) . "\n");
	printf ($OUTPUT "$storm \n");
	printf ($OUTPUT substr($storm, 2, 2) . "," . substr($aOrderedDTG[0], 0, 4) . ","  . substr($aOrderedDTG[0], 4, 2) . ","  . substr($aOrderedDTG[0], 6, 2) . ","    . substr($aOrderedDTG[0], 8, 2) . ","   . @aOrderedDTG . "\n\n");
	
	
	for (my $i = 0; $i < @aOrderedDTG; $i++) {
	
		my $key = $aOrderedDTG[$i];
	
		chop($hBtkValues{$key}->{latitude});
		chop($hBtkValues{$key}->{longitude});
	
		$hBtkValues{$key}->{longitude} = "-" . $hBtkValues{$key}->{longitude};
		printf ($OUTPUT "%6.3f  %7.3f %5.3f\n", $hBtkValues{$key}->{latitude}/10, $hBtkValues{$key}->{longitude}/10, $hBtkValues{$key}->{intensity});
	}
	
	close $OUTPUT;


}


#----------------------------------------------------------
# Subroutine: makeGrid
#
# Purpose: Create the RCLIPER grid files for nmap
#----------------------------------------------------------
sub makeGrid {

	#========================
	#*** Define Variables
	#========================
	

	my ($rcliper, $plot, $LOG) = @_;
	
	
	#==========================================
	#  *** Create GEMPAK GRID ***
	#=========================================
	
	
	#--- Print header to LOG
	
	generalUtils::header ("Create Grid with GEMPAK", $LOG);
		
	#--- Create the GEMPAK grid
		
	if ($plot->plotRcliper($rcliper, "grid", $LOG) != 1) {
		
		printf ($LOG "Error: Creating GEMPAK grid.\n");
		return;
		
	}
		
		
	#==========================================
	#  *** Add the GRID file to the NMAP GRID ***
	#=========================================
	
	
	if ($Config::addGridNMAP == 1) {
		
		addGridtoNmap($rcliper, $plot, $LOG);
		
	} 
		
		
	#==========================================
	#--- Rename and store the individual grid if not added to NMAP composite
	#==========================================
		
			
	if ($Config::storeResults == 1 && $Config::createGrid == 1) {
			
		rename ("$Config::workDir/" . $plot->getGridName(), "$Config::resultDir/" . $plot->getGridName());
			
	}
	elsif ($Config::createGrid == 0) {
	
		unlink ("$Config::workDir/" . $plot->getGridName());
	
	}
	

}


#----------------------------------------------------------
#Subroutine: makePS
#
# Purpose: Create a postscript file of the rcliper text output
#----------------------------------------------------------
sub makePS {


	#========================
	#*** Define Variables
	#========================
	

	my ($rcliper, $plot, $LOG) = @_;
	
		
	#=====================================
	#*** Create the Postscript image
	#=====================================
	
	
	#--- Print header to LOG
	
	generalUtils::header ("Create PS with GEMPAK", $LOG);
	
	if ( $plot->plotRcliper($rcliper, "ps", $LOG) != 1) {
		
		printf ($LOG "Error: Problem creating PS file.\n");
		
	}	

		
	#=====================================================
	#*** Print file if new and $Config::printPS is set to 1
	#=====================================================
	
		
	if (!(-e "$Config::psResultDir/" . $plot->getPSname()) && $Config::printPS) {
	
		foreach my $print (@Config::printer) {
		
			#generalUtils::printFile($Config::workDir, $plot->getPSname(), $Config::printer);
			generalUtils::printFile($Config::workDir, $plot->getPSname(), $print);
	
		}
	}	
	
	#--- Store PS file
	
	if ($Config::storeResults == 1) {
				
		#rename ("$Config::workDir/" . $plot->getPSname() , "$Config::psResultDir/" . $plot->getPSname()) || 
		#printf ($LOG "Error copying $Config::workDir/" . $plot->getPSname() . " to $Config::psResultDir: $!\n");
		
		copy ("$Config::workDir/" . $plot->getPSname() , "$Config::psResultDir/" . $plot->getPSname()) || 
		printf ($LOG "Error copying $Config::workDir/" . $plot->getPSname() . " to $Config::psResultDir: $!\n");
		unlink ("$Config::workDir/" . $plot->getPSname());

	}

}

#----------------------------------------------------------
# Subroutine: addGridtoNmap
#
# Purpose: Adds a grid file to a NMAP grid file.
#----------------------------------------------------------
sub addGridtoNmap {


	#========================
	#*** Define Variables
	#========================
	

	my ($rcliper, $plot, $LOG) = @_;
	
	#---Find a composite grid file in storage directory
	
	#my $pattern = "rcliper_[0-9]*f[0-9]*_" . $rcliper->{stormID};
	#my $pattern = "rcliper_[0-9]*_" . $rcliper->{stormID} . "f" . $$object{forecastHour};
	my $pattern = "rcliper_[0-9]*_" . $rcliper->{stormID};
	
	my @files;
	my $nmapGridDir = $Config::nmapResultDir;
	
	
	#==========================================
	#	*** Find any existing NMAP grid files
	#=========================================
	
	
	if (!generalUtils::findFiles ($Config::nmapResultDir, $pattern, \@files) ) {
	
	
		if (!generalUtils::findFiles ($Config::workDir, $pattern, \@files) ) {
		
			printf ($LOG "A composite grid matching $pattern does not exist in $Config::nmapResultDir or $Config::workDir.\n"); 
				
			#--- Assume this is the first grid for a storm.  Rename the grid and store if
			#--- indicated in the config file.
				
			if ($Config::storeResults) {
				
				copy ("$Config::workDir/" . $plot->getGridName(), "$Config::nmapResultDir/" . $plot->getNmapName()) || 
				printf ($LOG "Error: Could not copy " . $plot->getGridName() . " to $Config::resultDir : $!\n");
			
			}
			else {
			
				copy ("$Config::workDir/" . $plot->getGridName(), "$Config::workDir/" . $plot->getNmapName()) || 
				printf ($LOG "Error: Could not copy " . $plot->getGridName() . " to $Config::workDir : $!\n");
			
			}
			
			return;
			
		}
		
		$nmapGridDir = $Config::workDir;
				
	}
	
	
	#==========================================
	#  *** Add GRID to preexisting NMAP grid
	#==========================================
		
			
	printf ($LOG "A composite grid file matching $pattern exists in $nmapGridDir.\n");
	
	if ($nmapGridDir eq $Config::nmapResultDir) {
		
		#rename ("$Config::nmapResultDir/$files[0]", "$Config::workDir/$files[0]") || printf ($LOG "Error copying $Config::resultDir/$files[0] to $Config::workDir: $!\n");
		
		copy ("$Config::nmapResultDir/$files[0]", "$Config::workDir/$files[0]") || printf ($LOG "Error copying $Config::resultDir/$files[0] to $Config::workDir: $!\n");
		unlink ("$Config::nmapResultDir/$files[0]");
		
		

	}
	
	if (generalUtils::isFile($Config::workDir, $files[0]) == 1) {
	
		$plot->addPlot($files[0], $LOG);
			
		if ($Config::storeResults) {
				
			#rename ("$Config::workDir/$files[0]", "$Config::nmapResultDir/$files[0]") || 
			#printf ($LOG "Error copying $Config::workDir/$files[0] to $Config::nmapResultDir: $!\n");

			copy ("$Config::workDir/$files[0]", "$Config::nmapResultDir/$files[0]") || 
			printf ($LOG "Error copying $Config::workDir/$files[0] to $Config::nmapResultDir: $!\n");
			unlink ("$Config::workDir/$files[0]");

		}
		
	}	


}

#--------------------------------------------------------
# Subroutine: setup	*** Uses all global variables
#
# Purpose: Calls the routine to set up the working directory 
#			and then makes sure that all necessary files are 
#			copied to the working directory.
#
# Inputs: Reference to a IO file handle
#-----------------------------------------------------------
sub setup {


	#==========================
	#*** Define Variables
	#==========================
	

	my ($rObject, $rLOG) = @_;
	
	
	#===================================
	# *** Set up working environment  ***
	#===================================


	#--- Create the Work Directory, check lock file, open log

	setupEnv ($rObject, $rLOG);


	#==========================================
	#  *** Copy data files to working directory ***
	#=========================================


	#--- copy the coast line coordinate file

	copy ("$Config::dataDir/$coastlineFile", "$Config::workDir/$coastlineFile") ||
	fatal("Can't copy $Config::dataDir/$coastlineFile to $Config::workDir:", $!, $$rLOG);

	#--- copy the rcliper executable file 
	
	copy ("$Config::binDir/$$rObject{algorithm}", "$Config::workDir/$$rObject{algorithm}") ||
	fatal("Can't copy  $Config::binDir/$$rObject{algorithm} to $Config::workDir:", $!, $$rLOG);
	
	chmod(0744, "$Config::workDir/$$rObject{algorithm}") ||
	fatal("Can't make $Config::workDir/$$rObject{algorithm} executable:", $!, $$rLOG);

}


#--------------------------------------------------------
# Subroutine: setupEnv	*** Uses all global variables
#
# Purpose: Sets up the environment needed to run this program including:
#				- Checking for the existance of the working directory.
#				  If the working directory does not exist then create
#				  create one.
#				- Checking for the existance of a lock file.  If a lock file already
#				  exists then return and error message and end.  If a lock file
#				  does not exist then create one.
#				- Open a log file
#
# Inputs: Reference to a IO file handle
#--------------------------------------------------------
sub setupEnv {


	#=============================
	#*** Define Variables
	#=============================
	

	my ($rObject, $rLOG) = @_;
	my $returnCode;
	
	
	#===================================
	# *** Create WORK Directory ***
	#===================================


	#--- Create the Work Directory

	if (!(-d $Config::workDir)) {
		system ("mkdir $Config::workDir") && die "Can't Open $Config::workDir: $!";
	}
	

	#--- Check for existance of a lock file

	$returnCode = generalUtils::isFile($Config::workDir, $lockFile);

	if ($returnCode != -1) {
	
		#--- Check age of lock file ... if the lock file is older then some value
		#--- in minutes then assume this process died the last time it was
		#--- run.  Delete the lock file and run this program.
		
		if (generalUtils::oldFile($Config::workDir, $lockFile, 10) == 0) {
	
			printf ("Error ($returnCode): Lock directory exists. Program already running. \n");
			exit;
		
		}
		
		unlink ("$Config::workDir/$lockFile");
	
	}

	#--- Create a lock file
	
	system ("touch $Config::workDir/$lockFile");


	#=========================
	# *** Open Log file ***
	#=========================


	$returnCode = generalUtils::openFile($Config::workDir, $$rObject{processLog}, "write", $rLOG);
	
	if ($returnCode != 1) {
	
		printf ("Error ($returnCode): $Config::workDir/$$rObject{processLog} could not be opened for writing.\n");
	
	}
	
}

#--------------------------------------------------------
# Subroutine: fatal
#
# Purpose: Prints an error message to the log, cleans up the
#		   $working directory and end this program. 
#
# Inputs: $_[0] - string
#		  $_[1] - function string
#		  $_[2] - LOG
#--------------------------------------------------------
sub fatal{

	#--------------------------
	#*** Define Variables
	#--------------------------
	
	my($error, $functionError, $LOG) = @_;
	
	printf ($LOG "Fatal Error: $error $functionError");
	clean();
	die;

}

#--------------------------------------------------------
# Subroutine: clean		*** Uses all global variables
#
# Purpose: Delete intermediate files used during processing
#			the rcliper data
#--------------------------------------------------------
sub clean {

	unlink ("$Config::workDir/$coastlineFile");
	unlink ("$Config::workDir/$rcliperExe");
	unlink ("$Config::workDir/$lockFile");
	unlink ("$Config::binDir/gemglb.nts");
	unlink ("$Config::binDir/last.nts");
	system ("rm -f $Config::workDir/*.dat");
	
}
