package rcliper_data;

#! /usr/bin/perl -w

use 5.008;
use strict;
use IO::File;

require "rcliper_config.pl";
require $Config::generalCode;

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Program Name: rcliper_data.pl
#
# Version - 3.1
#
# Language: Perl 5.8.x
#
# Purpose:  This perl script uses the text result file produced by the RCLIPER
#	    algorithm and creates a data file in a format that can be used
#	    with GEMPAK to create/plot the RCLIPER results.
#
# Called by: rcliper_v3.pl
#---------------------------------------------------
# Data Structure
#
#		"textFile"		=> "",
#		"dataFile" 		=> "",
#		"stormName" 		=> "",
#		"stormID" 		=> "",
#		"trkPoints"	 	=> [],
#		"time" 			=> "",
#		"date" 			=> "",
#		"resolution" 		=> "",
#		"boundaries" 		=> "",
#		"parameter"		=> "RAIN"
#
#	stormName	The storm's assigned English name
#	stormID 	The ATCF id used for the storm (based on basin and storm number)
#	trkPoints   	Array containing previous storm track forecast points 
#	time 		The YYYYMMDDHH of the RCLIPER analysis
#	date 		The YYYYMMDD of the RCLIPER analysis
#	dataFile   	Data file containing RCLIPER algorithm text file output
#	boundaries  	The geographic extent of the RCLIPER result (used for the PS plot)
#	resolution  	The resolution of the RCLIPER result in lon/lat points
#	textFile    	Name of the RCLIPER output text file
#
#---------------------------------------------------
# Subroutines
#
### Subroutine: reformatText
#
# Purpose: Create the input files and initalize the variables 
#			needed as input to the GEMPAK functions.  Assign 
#			variable values based on information found in the file
#			name indicated by the user.  Create an output file
#			which can be used to plot the RCLIPER results with
#			GEMPAK.
#
### Subroutine: createFiles
#
# Purpose: Open the RCLIPER text output file passed into the function.  
#		   If the file is successfully opened for reading
#		   then read the information and store in the data structure.
#
#		   Create a new data file in a format that can be read
#			by the GEMPAK program ASCTOGEM.
#
### Subroutine: readLog
#
# Purpose: Reads the log file produced by the rcliper executable and
#			uses the information to populate RCLIPER data structure.
#
#------------------------------------------------------
# Access Subroutines
#
### Subroutine: getDate
#
# Purpose: Returns the date of the RLCIPER analysis
#			to the caller
#
### Subroutine: getTime
#
# Purpose: Returns the time of the RLCIPER analysis
#			to the caller
#
### Subroutine: getStormID
#
# Purpose: Returns the StormID of the tropical
#			cyclone being analyized to the caller
#
### Subroutine: getName
#
# Purpose: Returns the name of the storm
#			to the caller
#
### Subroutine: getTrack
#
# Purpose: Returns an array containing
#			the storm's ATCF track points
#		 	to the caller
#
### Subroutine: getExtent
#
# Purpose: Returns the number of points
#			in the x/y directions in the RCLIPER 
#			data file to the caller
#
### Subroutine: getBoundaries
#
# Purpose: Returns the lat/lon coordinates of
#			of the geographic boundries of the RCLIPER
#			analysis file to the caller
#
### Subroutine: getText
#
# Purpose: Returns the name of the RCLIPER text file 
#          created by the RCLIPER executable
#
#---------------------------------------------------
# Information:
#    Joint Hurricane Testbed/USWRP project
#    National Hurricane Center / Tropical Prediction Center
#    Department of Commerce - NOAA / NWS
#
# Category: JHT - RCLIPER 
#
# Created: Alison Krautkramer 10/2003
#
# Modification:
#
#	10/2004
#		
#		Modified program so that each forecast hour grid
#		can be added to one gempak grid file.
#
#	 	line 147 -> changed parameter list to include forecast time
#		line 153 -> added forecast time to data structure
#		line 476 -> print the forecast time into the text file
#			for conversion into a gempak grid
#
#	11/17/2004 - ARK
#
#		Modified program so that each forecast hour grid will be added
#		under a different parameter name.
#		
#		line 173 -> Removed to characters from the grid name because
#			the asctogem program only accomodates 4 character names
#			Therefore, the RAIN parameter became RN72, RN48 or RN24.
#		line 180 -> Appended the forecast time period to the gempak parameter name
#			to differential forecast periods 
#		line 674 -> Added getParameter() function to return the name
#			of the gempak parameter
#		
#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


#--------------------------------------------------------
# Subroutine: new
#
# Purpose: Initialize the RCLIPER data data structure
#
# Calls: readLog
#
# Inputs:
#		  $_[0] - filename of the RCLIPER log file
#		  $_[2] - log file handle
#
# Outputs: 1 - success
#
# Errors:  undef - error can't create rcliper data object
#-----------------------------------------------------
sub new {

	my ($infoFile, $forecast, $LOG) = @_;
	my $returnCode;

	my $rRcliper = {
					"textFile"		=> "",
					"dataFile" 		=> "",
					"forecast"		=> $forecast,
					"stormName" 	=> "",
					"stormID" 		=> "",
					"trkPoints"	 	=> [],
					"time" 			=> "",
					"date" 			=> "",
					"resolution" 	=> "",
					"boundaries" 	=> "",
					#"parameter"		=> "RAIN"
					"parameter"		=> "RN"
					
			      };
			      
	$$rRcliper{parameter} = $$rRcliper{parameter} . $$rRcliper{forecast};

	bless $rRcliper, "rcliper_data";
	
	if ($rRcliper->readLog($infoFile, $LOG) != 1) {
	
		printf ($LOG "Error: Could not successfully read RCLIPER log. \n");
		return undef;
		
	}

	return $rRcliper;

}




#--------------------------------------------------------
# Subroutine: reformatText
#
# Purpose: Create the input files and initalize the variables 
#			needed as input to the gempak functions.  Assign 
#			variable values based on information found in the file
#			name indicated by the user.  Create an output file
#			which can be used to plot the RCLIPER results with
#			GEMPAK.
#
# Calls: createFiles
#
# Inputs: $_[0] - RCLIPER data object
#		  $_[1] - log file handle
#
# Outputs: 1 - success
#
# Errors:  -1 Incorrect number of parameters in the file
#			- Open file error code
#-----------------------------------------------------
sub reformatText {


	#-----------------------
	# **** Define Variables
	#-----------------------
	
	
	my ($rRcliper, $LOG) = @_;
	my $returnCode;


	#-------------------------------------------------
	# *** Create RCLIPER GEMPAK data file from the RCLIPER text file
	#-------------------------------------------------


	$returnCode = createFiles($rRcliper, $LOG);
	
	if ($returnCode != 1) {
	
		printf ($LOG "Error ($returnCode): From creating the data and track files.\n");
		return $returnCode;
	
	}


	#---------------------------------------------
	# *** Check for the existance of the newly created RCLIPER GEMPAK data file
	#---------------------------------------------


	$returnCode = generalUtils::isFile($Config::workDir, $$rRcliper{dataFile});

	if ($returnCode != 1) {

		printf ($LOG "Error ($returnCode) : $$rRcliper{dataFile} failed to be created or has zero size.\n");
		return -1;	#no data file created

	}
	
	return 1;
	
}

#--------------------------------------------------------
# Subroutine: createFiles
#
# Purpose: Open the RCLIPER text output file passed into the function.  
#		   If the file is successfully opened for reading
#		   then read the information and store in variables.
#
#		   Create a new data file in a format that can be read
#			by the GEMPAK program ASCTOGEM.
#
# Expected input RCLIPER text file:
#
#			EP142003 100812 NORA
# 			121    0.250 -122.000  -92.000 121    0.250   13.000   43.000
# 			-108.00  21.00   0.0
# 			-106.74  22.29  12.0
# 			-107.65  23.52  24.0
# 			-108.56  24.49  36.0
#   			-0.00   0.00  48.0
#  		    	-0.00   0.00  60.0
#   			-0.00   0.00  72.0
# 			-122.000   13.000    0.000
# 			-121.750   13.000    0.000
# 			-121.500   13.000    0.000		...etc
#
#	Line 1:  Contains the name and DTG of the storm 
#				
#				example: storm number: EP142003
#						 date: 1008
#						 time: 12
#						 storm name: NORA
#
#	Line 2: Contains information about the resolution of the file.
#
#				example: resolution: 0.25 0.25
#						 number of data points in x/y direction: 121 121
#						 boundary coordinates:  -122.000  -92.000 13.000   43.000
#
#	Line 3 - 10: Contain the 0-72 hour (12 hour increment) 
#				 track forecast positions 
#
#					example: 0 hour: -108.00  21.00 
#							 12 hour: -106.74  22.29
#
#	
#	Line 11 - n: Contains the rain amount for at a lat/lon point.
#
# Output Data file format:
#
#		031008/1200  0  RAIN
#		13.000 -122.000 0.000
#		13.000 -121.750 0.000
#		13.000 -121.500 0.000
#		13.000 -121.250 0.000
#		13.000 -121.000 0.000  	...etc
#
#	Line 1: Contains YYMMDD/HH00 0 RAIN
#
#	Line 2-n: Contains lat/lon/rain amount in inches 
#			  The values are listed from the upper left
#			  corner of the result grid to the lower right corner.
#		
#
# Called by: reformatText
#
# Inputs: $_[0] - RCLIPER data object
#		  $_[1] - log file handle
#
# Outputs: 1 - success
#
# Errors:  -1 Incorrect number of parameters in the file
#			- Open file error code
#--------------------------------------------------------
sub createFiles {

	#----------------------------
	# *** Define local variables
	#----------------------------
	
	my ($rRcliper, $LOG) = @_;
	
	my @lat = (); 
	my @lon = (); 
	my @rain = ();
	
	#--- Variables
	
	my @parsedLine = "";
	my $line;
	my $returnCode;
	
	#--- File Handles
	
	my $INPUT = new IO::File;
	my $DATA = new IO::File;
	

	#---------------------------------------------------
	# ***  Open the file for reading ***
	#---------------------------------------------------
	
	
	$returnCode = generalUtils::openFile ($Config::workDir, $$rRcliper{textFile}, "read", \$INPUT);
	
	if ($returnCode != 1) {
	
		printf ($LOG "Error ($returnCode): Failed opening $$rRcliper{textFile}\n");
		return $returnCode;
		
	}
	
	
	#---------------------------------------------------
	# ***  Read Input file ***
	#---------------------------------------------------
	
	
	#---------------------------------------------------
	#Line 1:  Contains the name and DTG of the storm
	#--- year is the last two digits
	#--- 4 digit MMDD 
	#HH of the storm
	#----------------------------------------------------
		
	if (!defined($line = <$INPUT>)) {
		
		return -1;	# Error no line found
		
	}
	
	
	#---------------------------------------------------	
	#Line 2: Contains information about the resolution of the file.
	#---------------------------------------------------	
	
	if (defined($line = <$INPUT>)) {
		
		chomp $line;
		
		@parsedLine = split (/\s+/, $line);
		
		if (@parsedLine != 9) {
		
			printf ($LOG "Error: Incorrect number of parameters in the resolution line.\n");
			return -1;
			
		}
				
		$$rRcliper{resolution} = "$parsedLine[1];$parsedLine[5]";
		$$rRcliper{boundaries} = "$parsedLine[7];$parsedLine[3];$parsedLine[8];$parsedLine[4]";
		
	}
	
	
	#-------------------------------------------------------------------------	
	#Line 3 - 10: Contain the track forecast positions 
	#-------------------------------------------------------------------------
			
	 for (my $i = 0; $i < @{$$rRcliper{trkPoints}}; $i++) {
		
		if (defined($line = <$INPUT>)) {
			
			#chomp $line;
			
			#@parsedLine = split (/\s+/, $line);
			
			#if (@parsedLine != 4) {
			
			#	printf ($LOG "Error: Incorrect number of parameters in the track line.\n");
			#	return -1;
				
			#}
			
			#--- Only include the non-zero track points
				
			#if ((abs($parsedLine[1]) > 0.001) &&  (abs($parsedLine[2]) > 0.001)) {
			
			#	$$rRcliper{trkPoints}->[$i] = "$parsedLine[2] $parsedLine[1]";
				
			#}	
		}
				
	}
		
		
	#---------------------------------------------------------------	
	#Line 11 - n: Contains the rain amount for at a lat/lon point.
	#----------------------------------------------------------------		
		
	while (defined($line = <$INPUT>)) {
		
		chomp ($line);
		
		@parsedLine = split (/\s+/, $line);
		
		if (@parsedLine != 4) {
		
			printf($LOG "Error: Incorrect number of parameters.\n");
			return -1;
			
		}
			
		$lon[@lon] = $parsedLine[1];
		$lat[@lat] = $parsedLine[2];
		$rain[@rain] = $parsedLine[3];
		
	}
		
	close $INPUT;
	
	
	#---------------------------------------------------
	# ***  Write Output DATA File ***
	#---------------------------------------------------
	
	
	#--- Open a file to write the lat/lon/rain data into.
	
	$returnCode = generalUtils::openFile ($Config::workDir,$$rRcliper{dataFile} , "write", \$DATA);
	
	if ($returnCode != 1) {
		
		printf ($LOG "Error ($returnCode): failed opening rainplt.dat\n");
		return $returnCode;
		
	}

	#--- Write the information into the data file
	
	printf ($DATA  substr($$rRcliper{time}, 2, 6) . "/" . substr($$rRcliper{time}, 8, 2) . "00  0  " . $$rRcliper{parameter} . "\n");
	#printf ($DATA  substr($$rRcliper{time}, 2, 6) . "/" . substr($$rRcliper{time}, 8, 2) . "00F0" . $$rRcliper{forecast} . "  0  " . $$rRcliper{parameter} . "\n");
	
	for (my $i = 0; $i < @lat; $i++) {
	
		printf $DATA "$lat[$i] $lon[$i] $rain[$i] \n";
		
	}
	
	close $DATA;
	
	return 1;

}


#--------------------------------------------------------
# Subroutine: readLog
#
# Purpose: Reads the log file produced by the rcliper executable and
#			uses the information to populate RCLIPER data variables.
#
#	example: more rcliper.log
#
#		RCLIPER forecast for AL132003 060318 LILI
#
# 			Track/Intensity
#  			  0.  13.2   66.0  60.
#  			 12.  14.2   68.5  65.
# 			 24.  15.3   70.2  70.
# 			 36.  16.9   71.7  78.
#  			 48.  18.5   73.3  78.
#  			 60.  19.7   74.4  68.
#  			 72.  20.8   75.0  66.
#
# 			Track forecast out to  72. hrs
#
# 			Divide track into 145 points 0.5 hrs apart
#
# 			Storm inland at t= 45.0 lon/lat:   -72.9   18.2  dtl=  -1.9
#
# 			Rainfall grid reference lon/lat:   -73.0   18.0
#
# 			Max rain= 6.7 at lon/lat:  -74.000   19.500
#
# 			Output file name: 03060318AL13_rcliper.txt
#		 
# Called by: new (class constructor)
#
# Inputs: $_[0] - RCLIPER data object
#		  $_[1] - name of RCLIPER log file
#		  $_[2] - LOG file handle
#
# Outputs: 1 - success	  
#
# Errors:  -1 - Incorrect $rcliperLog format
#--------------------------------------------------------
sub readLog {


	#-----------------------------
	#  *** Define local variables
	#-----------------------------
	
	
	my ($rRcliper, $rcliperLog, $LOG) = @_;
	
	my $INPUT = new IO::File;
	my $found = 0;
	
	
	#--------------------------
	# *** Open Rcliper Log
	#--------------------------
	
	
	my $returnCode = generalUtils::openFile($Config::workDir, $rcliperLog, "read", \$INPUT);
	
	if ($returnCode != 1) {
	
		printf ($LOG "Error ($returnCode): $Config::workDir/$rcliperLog could not be opened for reading.\n");
		return $returnCode;
	
	}
	
	
	#----------------------------------
	# *** Extract information from LOG
	#-----------------------------------
	
	
	while (defined(my $line = <$INPUT>)) {
	
		my @parsedLine = ();
		
		if ($line =~ /RCLIPER forecast for/) {
		
			chomp $line;
			
			@parsedLine = split(/\s+/, $line);
			
			$$rRcliper{stormID} = substr($parsedLine[4], 0, 8);
			$$rRcliper{stormID} =~ tr/A-Z/a-z/;
			
			$$rRcliper{date} = substr($parsedLine[4], 4, 4) . substr($parsedLine[5], 0, 4) ;
			
			$$rRcliper{time} = $$rRcliper{"date"} . substr($parsedLine[5], 4, 2);
			
			$$rRcliper{stormName} = $parsedLine[6];
			
			$found++;
		
		}
		
		if ($line =~ /Track\/Intensity/) {
					
			while (defined($line = <$INPUT>) && ($line =~ /[0-9]/)) {
			
				chomp $line;
		
				@parsedLine = split (/\s+/, $line);
				
				if (@parsedLine != 5) {
			
					printf ($LOG "Error: Incorrect number of parameters in the track line.\n");
					return -1;
				
				}
				
				$$rRcliper{trkPoints}->[@{$$rRcliper{trkPoints}}] = "$parsedLine[2] -$parsedLine[3]";
				#$$rRcliper{trkPoints}->[@{$$rRcliper{trkPoints}}] = "0 0";
		
			}
		}
		
		
		if ($line =~ /Output file name:/) {
		
			chomp $line;
			
			@parsedLine = split(/:/, $line);
			
			$parsedLine[1] =~  s/\s+//g;
			$$rRcliper{textFile} = $parsedLine[1];
			$parsedLine[1]  =~ s/txt/data/;
			$$rRcliper{dataFile} = $parsedLine[1];
			
			$found++;
		
		}
	
	}
	
	close $INPUT;
	
	if ($found != 2) {
	
		printf ($LOG "Error ($found): " . $rcliperLog . "'s format is incorrect.\n");
		return -1; 
	
	}
	
	return 1;
}


#--------------------------------
#
#	Accessor methods
#
#--------------------------------


#------------------------------------
# Subroutine: getParameter
#
# Purpose: Returns the date of the RLCIPER analysis
#			to the caller
#------------------------------------
sub getParameter {

	my ($rRcliper) = @_;
	
	return $rRcliper->{parameter};

}



#------------------------------------
# Subroutine: getDate
#
# Purpose: Returns the date of the RLCIPER analysis
#			to the caller
#------------------------------------
sub getDate {

	my ($rRcliper) = @_;
	
	return $rRcliper->{date};

}

#------------------------------------
# Subroutine: getTime
#
# Purpose: Returns the time of the RLCIPER analysis
#			to the caller
#------------------------------------
sub getTime {

	my ($rRcliper) = @_;
	
	return $rRcliper->{time};

}

#------------------------------------
# Subroutine: getStormID
#
# Purpose: Returns the StormID of the tropical
#			cyclone being analyized to the caller
#------------------------------------
sub getStormID {

	my ($rRcliper) = @_;
	
	return $rRcliper->{stormID};

}

#------------------------------------
# Subroutine: getData
#
# Purpose: Returns the name of the data file 
#          used to create the GEMPAK grids 
#		   to the caller
#------------------------------------
sub getData {

	my ($rRcliper) = @_;
	
	return $rRcliper->{dataFile};

}

#------------------------------------
# Subroutine: getText
#
# Purpose: Returns the name of the RCLIPER text file 
#          created by the RCLIPER executable
#------------------------------------
sub getText {

	my ($rRcliper) = @_;
	
	return $rRcliper->{textFile};

}

#------------------------------------
# Subroutine: getName
#
# Purpose: Returns the name of the storm
#			to the caller
#------------------------------------
sub getName {

	my ($rRcliper) = @_;
	
	return $rRcliper->{stormName};

}

#------------------------------------
# Subroutine: getTrack
#
# Purpose: Returns an array containing
#			the storm's ATCF track points
#		 	to the caller
#------------------------------------
sub getTrack {

	my ($rRcliper) = @_;
	
	return $rRcliper->{trkPoints};

}

#------------------------------------
# Subroutine: getExtent
#
# Purpose: Returns the number of points
#			in the x/y directions in the RCLIPER 
#			data file to the caller
#------------------------------------
sub getExtent {

	my ($rRcliper) = @_;
	
	return $rRcliper->{resolution};

}

#------------------------------------
# Subroutine: getBoundaries
#
# Purpose: Returns the lat/lon coordinates of
#			of the geographic boundries of the RCLIPER
#			analysis file to the caller
#------------------------------------
sub getBoundaries {

	my ($rRcliper) = @_;
	
	return $rRcliper->{boundaries};

}



1;
