package atcfStorms;

#! /usr/bin/perl -w

use strict;
use Date::Manip;
use IO::File;

require "generalUtils_v1.pl";

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# PACKAGE NAME: atcfStorms 
#
# PURPOSE: Provides a variety of subroutines to
#
#			1. Identify the tropical cyclones active during a date/time
#			2. Create a computes file for an active tropical cyclone
#			3. Find the start/end dates of a tropical cylone
#
# LANGUAGE: Perl 5.8.x
#
# USAGE:
#	
#	* To create a atcfStorms Object
#	
#		atcfStorms::new(ATCF_directory, YYYYMMDDHH); 
#			
#		exmaple: my $atcf = atcfStorms::new("/data/atcf", "2003090812");
#
#	* To get a list of active storms
#
#		example: my $raStorms = $atcf->getActiveStorms();
#
#	* To get the date/time group the object was initalized with
#
#		example: my $date = $atcf->getDate();
#
#	* To get the OFCL/CARQ records for a the storm/date 
#
#		example: $atcf->computes(); #get computes for all active storms
#							
#				-or-
#							
#				 $atcf->stormComputes("al132003"); # get computes for 1 active storm
#
#	* To create a computes file
#
#		atcfStorms->createStormComputes (stormName, destination_directory, computes_fileName);
#		atcfStorms->createComputes()
#
#		example: $atcf->createStormComputes("al132003", "/home/akrautkr","comab.dat");
#							
#				-or-
#							
#	   		     $atcf->createComputes(); # create files for all storms with default file name
#
#
# INPUT FILES: ATCF best track files
#		ATCF aids files
#		ATCF computes files
#
# OUTPUT FILES: Computes files either named a default name of stormname.com
#				or a user defined name
#
# Called by:  rcliper_v6.pl
#---------------------------------------------------
# DATA STRUCTURE:
#
#			 {
#	
#					"date" => Date::Manip::ParseDate($date),
#					"atcfDir" => $atcfDir,
#					"storms"	=> [],
#					"computes" => {}
#				
#			 };
#
#	date - date/time group
#	atcfDir - directory of the atcf root directory
#	stroms - array of strings representing storms active during date
#	computes - hash containing
#
#					=> OFCL []	- array of ATCF OFCL records 
#					=> CARQ []	- array of ATCF CARQ records
#					=> startDate - Date::Manip object representing the start of the storm
#					=> endDate 	-  Date::Manip object representing the end of the storm
#
#---------------------------------------------------
# BRIEF DESCRIPTION OF PROGRAM SUBROUTINES:
#
#	new - Initialize the ATCF data object.
#	
#	activeStorms - Finds all of the storms active during the
#				   calendar date/time used to initialize the
#				   ATCF object.  The list of active storms are
#				   stored in the ATCF object's storms array.
#	
#	computes -   For each storm in the storms array find the 
#				 CARQ and OFCL ATCF records for the appropriate
#				 date/time periods.  The date/time periods
#				 are calculated using the date/time the ATCF
#				 object was initilized at.  
#	
#	stormComputes - For a specific storm in the storms array find the 
#				    CARQ and OFCL ATCF records for the appropriate
#				    date/time periods.  The date/time periods
#				    are calculated using the date/time the ATCF
#				    object was initilized at.  
#
#	readATCFfiles -  Read the correct ATCF file for the particular
#				     storm and type (com or dat).  Read the ATCF file
#				     into an array to identify any CARQ/OFCL records.
#				     If records are found then add them to the ATCF object.
#
#	createComputes - Creates a computes file for each active storm.
#				 	 The computes file is named in a standard default,
#				 	 stormName.com. 
#
#	createStormComputes - Creates a computes file for a particular active storm.
#
#	getComputes - Examines an array of ATCF records and identifies
#				  and individual records CARQ or OFCL records 
#				  for the appropriate date/time groups.  The 
#				  appropriate date/time groups are calculated using
#				  the date/time stored in the ATCF object.
#
#	stormDuration - Opens and reads through a best track file for a 
#				    particular storm and extracts the date of the first
#				    best track record and the date of the last best
#				    track record.  The start and end dates are returned
#
#	getDTG -  Takes a ATCF format string and parses
#			  out the date/time group field.  The DTG
#			  is passed back to the caller in teh $rDate 
#			  variable.
#
#	getDate - Returns the calendar date that the ATCF Object
#				 was initilized with.  This calendar date is 
#				 used to identify a list of active storms and 
#				 then create the computes files.  
#
#	getActiveStorms - Returns a reference to an array that contains
#				 	  a list of the active storms.  Active storms are
#				 	  defined as any tropical cyclones in which best track
#				 	  records are being written during the date specified
#				 	  by the user.  The date is specified when the
#				 	  ATCF object is initialized.
#---------------------------------------------------
# Information:
#    Joint Hurricane Testbed/USWRP project
#    National Hurricane Center / Tropical Prediction Center
#    Department of Commerce - NOAA / NWS
#
# Category: JHT - ATCF
#
# Created: Alison Krautkramer 11/2003
#
# Modified: 3/2004 
#		Prepared code for transfer to operations
#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-



#***********************************************************
#
#	SUBROUTINE NAME: new
#
#	DESCRIPTION: Initialize the ATCF data object.
#
#	USAGE (CALLING SEQUENCE):external initlization
#
#	VARIABLES AND THEIR USE
#		PASSED:	ATCF root directory
#				A date/time group in YYYYMMDDHH format
#
#		RETURNED: An ATCF data Object
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub new {

	#--- Initialize variables

	my ($atcfBtkDir, $atcfComDir, $atcfAidDir, $date) = @_;
	
	#--- Check values
	
	my $returnCode = generalUtils::validDateFormat($date);
	
	if ($returnCode != 1) {
	
		printf ("Error $returnCode (atcfStorms::new): Date must be a string in YYYYMMDDHH format.\n");
		return undef;
	
	}
	
	unless (-d $atcfBtkDir && -d $atcfComDir && -d $atcfAidDir) {
	
		printf ("Error (atcfStorms::new): One ATCF directory specified does not exist $atcfBtkDir $atcfComDir $atcfAidDir.\n");
		return undef;
	
	}
	
	#--- Initialize data structure
	
	my $rATCF = {
	
					"date" => Date::Manip::ParseDate($date),
					"atcfBtkDir" => $atcfBtkDir,
					"atcfAidDir" => $atcfAidDir,
					"atcfComDir" => $atcfComDir,
					"storms"	=> [],
					"computes" => {}
				
				};
				
	bless $rATCF, "atcfStorms";
	
	$returnCode = $rATCF->activeStorms();
	
	return $rATCF;
	
}



#***********************************************************
#
#	SUBROUTINE NAME: activeStorms
#
#	DESCRIPTION: Finds all of the storms active during the
#				 calendar date/time used to initialize the
#				 ATCF object.  The list of active storms are
#				 stored in the ATCF object's storms array.
#
#	USAGE (CALLING SEQUENCE): new()
#
#	VARIABLES AND THEIR USE
#		PASSED: ATCF object
#
#		RETURNED: none
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub activeStorms {

	#--- Initialize variable
	
	my ($atcfObj) = @_;
	
	#--- Emtpy the storms array if needed
	
	if (@{$$atcfObj{storms}} != 0) {
	
		@{$$atcfObj{storms}} = [];
	
	}
	
	#--- Get a list of all best track files
	
	my @btkFiles;
	
	#--- ATCF database structure
	
	#^b(ep|al|cp|wp)[^8][0-9]{5}\.dat ... start wtih b followed by either ep or al or cp or wp which is not an 80's 
	#storm and with 5 digits followed by the string .dat.
	
	my $returnCode = generalUtils::findFiles("$$atcfObj{atcfBtkDir}", "^b(ep|al|cp|wp)[^8][0-9]{5}\.dat", \@btkFiles);
	
	if ($returnCode != 1) {
		
		printf ("Error $returnCode (atcfStorms::activeStorms) : No best track files found in " . $$atcfObj{atcfBtkDir} .  ".\n");
		return $returnCode;	
	
	}
	
	
	#--- for each best track file find the storms begining and ending dates.
	#--- Compare the storm boundary dates to the date stored in the ATCF object.
	
	foreach my $file (@btkFiles) {
		
		my ($startDate, $endDate);
		if (stormDuration($file, "$$atcfObj{atcfBtkDir}", \$startDate, \$endDate) == 1) {
		
			if (Date::Manip::Date_Cmp($startDate,$$atcfObj{date}) <= 0 &&
				Date::Manip::Date_Cmp($endDate, $$atcfObj{date}) >= 0) {
		
				#--- Add storm name to storms array
				
				push(@{$$atcfObj{storms}}, substr($file, 1, 8));
				
				#--- Add storm information to computes array
				
				$$atcfObj{computes}->{substr($file, 1, 8)}->{startDate} = $startDate;
				$$atcfObj{computes}->{substr($file, 1, 8)}->{endDate} = $endDate;
				$$atcfObj{computes}->{substr($file, 1, 8)}->{ofcl} = [];
				$$atcfObj{computes}->{substr($file, 1, 8)}->{carq} = [];
		
			}
		}

	}
}



#***********************************************************
#
#	SUBROUTINE NAME: computes
#
#	DESCRIPTION: For each storm in the storms array find the 
#				 CARQ and OFCL ATCF records for the appropriate
#				 date/time periods.  The date/time periods
#				 are calculated using the date/time the ATCF
#				 object was initilized at.  
#
#	USAGE (CALLING SEQUENCE): external
#
#	VARIABLES AND THEIR USE
#		PASSED: ATCF Object
#
#		RETURNED: 0 no current storms found in the storms array
#				  1 success
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub computes {

	#--- Initialize variable
	
	my ($atcfObj) = @_;

	foreach my $storm (@{$$atcfObj{storms}}) {
	
		$atcfObj->stormComputes($storm);
		return 1;
	
	}
	
	#--- No storms listed in the storms array
	
	return 0;

}



#***********************************************************
#
#	SUBROUTINE NAME: stormComputes
#
#	DESCRIPTION: For a specific storm in the storms array find the 
#				 CARQ and OFCL ATCF records for the appropriate
#				 date/time periods.  The date/time periods
#				 are calculated using the date/time the ATCF
#				 object was initilized at.  
#
#	USAGE (CALLING SEQUENCE): computes() or external
#
#	VARIABLES AND THEIR USE
#		PASSED: ATCF object
#				Storm name
#
#		RETURNED: -1 no CARQ or OFCL records found
#				   1 success
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub stormComputes {

	#--- Initialize variable
	
	my ($atcfObj, $storm) = @_;
	
	
	#--- Check the computes directory for a computes file.  If
	#--- one is not found, check the appropriate A deck file for 
	#--- CARQ/OFCL records.
	
	if ($atcfObj->readATCFfiles($storm, "com") != 1) {
	
		if ($atcfObj->readATCFfiles($storm, "aid") != 1) {
		
			printf ("Error (atcfStorms::stormComputes): No CARQ or OFCL records found for $storm.\n");
			return -1;
		}
	
	}
	
	return 1;	#computes found

}



#***********************************************************
#
#	SUBROUTINE NAME: readATCFfiles
#
#	DESCRIPTION: Read the correct ATCF file for the particular
#				 storm and type (com or dat).  Read the ATCF file
#				 into an array to identify any CARQ/OFCL records.
#				 If records are found then add them to the ATCF object.
#
#	USAGE (CALLING SEQUENCE): stormComputes()
#
#	VARIABLES AND THEIR USE
#		PASSED: ATCF storm object
#				name of a storm
#				type of ATCF file to examine (com -> computes dat -> adeck)
#
#		RETURNED: -1 command syntax error
#				  -2 .. -5 open file error
#				  -3 OFCL/CARQ records not found for date/time
#				   1 success
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub readATCFfiles {

	#--- Initialize variable
	
	my ($atcfObj, $storm, $type) = @_;
	my $INPUT = new IO::File;
	my $pattern;
	my $dir;
	
	#--- Check $type parameter
	
	if ($type eq "aid") {
	
		$pattern = "a$storm.dat";
		$dir = $$atcfObj{atcfAidDir};
	
	}
	elsif ($type eq "com") {
	
		$pattern = "$storm.com";
		$dir = $$atcfObj{atcfComDir};
		
	}
	else {
	
		printf ("Error (atcfStorms::readATCFfiles): syntax $atcfObj->readATCFfiles(storm, <aid|com>)\n");
		return -1;
	
	}
	
	#--- Open file containing the storm number
	
	my $returnCode = generalUtils::openFile($dir, $pattern, "read", \$INPUT);
	
	if ($returnCode != 1) {
	
		printf ("Error $returnCode (atcfStorms::readATCFfiles): Error opening $dir/$pattern. \n");
		return $returnCode;
	
	}
	
	#--- Determine whether the OFCL and CARQ records for the correct time period
	
	my @lines = <$INPUT>;
	close ($INPUT);
	
	my @OFCL;
	my @CARQ;
	
	#--- If success then add arrays to data structure and return 1
	
	if (getComputes(\@lines, $$atcfObj{date}, \@OFCL, \@CARQ) == 1) {
	
		$$atcfObj{computes}->{$storm}->{ofcl} = \@OFCL;
		$$atcfObj{computes}->{$storm}->{carq} = \@CARQ;
		
		return 1;
	
	}
	
	return -3;	#Failure


}



#***********************************************************
#
#	SUBROUTINE NAME: createComputes
#
#	DESCRIPTION: Creates a computes file for each active storm.
#				 The computes file is named in a standard default,
#				 stormName.com.  
#
#	USAGE (CALLING SEQUENCE): external
#
#	VARIABLES AND THEIR USE
#		PASSED: ATCF object
#				Directory to place the new computes file
#
#		RETURNED: Creates a new computes file
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub createComputes {

	#--- Initialize variable
	
	my ($atcfObj, $dir) = @_;
	
	foreach my $storm (@{$$atcfObj{storms}}) {
	
		if ($atcfObj->createStormComputes($storm, $dir, "$storm.com") != 1) {
		
			printf ("No computes created for $storm.\n");
		
		}
	
	}


}



#***********************************************************
#
#	SUBROUTINE NAME: createStormComputes 
#
#	DESCRIPTION: Creates a computes file for a particular active storm.
#				
#
#	USAGE (CALLING SEQUENCE):createComputes() or external
#
#	VARIABLES AND THEIR USE
#		PASSED: ATCF Object
#				storm name as a string
#				directory for the computes file
#				file name for the computes file
#
#		RETURNED: -1 Storm name not found in ATCF object
#				  -2 Problem creating the computes file
#				  -3 No CARQ or OFCL records found for this storm
#				   1 success
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub createStormComputes {

	#--- Initialize variable
	
	my ($atcfObj, $storm, $dir, $filename) = @_;
	my $OUTPUT = new IO::File;
	
	#---- Check the storm is contained in the computes hash
	
	if (!exists($$atcfObj{computes}->{$storm})) {
	
		printf ("Error (atcfStorms::createStormComputes): Storm $storm not found in computes hash.\n");
		return -1;
	
	}
	
	#--- Open a file for writing computes to
	
	if (@{$$atcfObj{computes}->{$storm}->{ofcl}} > 0 && 
		@{$$atcfObj{computes}->{$storm}->{carq}} > 0) 
	{
		printf ("create computes file");
		my $returnCode = generalUtils::openFile($dir, $filename, "write", \$OUTPUT);
	
		if ($returnCode != 1) {
	
			printf ("Error $returnCode (atcfStorms::createStormComputes): $dir/$filename could not be opened for writing \n");
			return $returnCode;
	
		} 
	
		print ($OUTPUT @{$$atcfObj{computes}->{$storm}->{ofcl}});
		print ($OUTPUT @{$$atcfObj{computes}->{$storm}->{carq}});
		close $OUTPUT;
		return 1;
	
	}
	
	return -3;

}



#***********************************************************
#
#	SUBROUTINE NAME: getComputes
#
#	DESCRIPTION: Examines an array of ATCF records and identifies
#				 and individual records CARQ or OFCL records 
#				 for the appropriate date/time groups.  The 
#				 appropriate date/time groups are calculated using
#				 the date/time stored in the ATCF object.
#
#	USAGE (CALLING SEQUENCE): readATCFfiles()
#
#	VARIABLES AND THEIR USE
#		PASSED: Reference to an array of ATCF records
#				DTG in YYYYMMDDHH format
#				Reference to an empty array for OFCL records
#				Reference to an empty array for CARQ records
#
#		RETURNED: Array filled with OFCL records that match the desired DTG
#				  Array filled with CARQ records that match the desired DTG
#				  -1 Error finding the synoptic CARQ and OFCL times
#				  -2 No OFCL or CARQ records found
#				   1 Success
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************

sub getComputes {

	#--- Initialize variable
	
	my ($raRecords, $date, $raOFCL, $raCARQ) = @_;
	my $synopticDate;
	my $synopticDateM6;
	
	#--- Convert $date to a string
	
	my $stringDate = Date::Manip::UnixDate($date, '%Y%m%d%H');
	
	#--- Find synoptic times
	
	if ((generalUtils::synopticTime($stringDate, 0, \$synopticDate) != 1) ||
		(generalUtils::synopticTime($stringDate, -1, \$synopticDateM6) != 1))
	{

		print ("Error (atcfStorms::getComputes) finding the current synoptic time for $date");
		return -1;
		
	}
	
	#--- Search array
	
	foreach my $line (@{$raRecords}) {
	
		#--- Check line for the string 'CARQ' and 'synopticDTG'
	
		if (($line =~ /CARQ/) && ($line =~ /$synopticDate,/)) {
		
			#if ($line =~ /,\sCARQ,\s+0,/) {
			if ($line =~ /,\sCARQ,\s+/) {
			
				push (@{$raCARQ}, $line);
				
			}
		
		}
		
		#--- Check line for the string 'CARQ' and 'previous synopticDTG'
		
		if (($line =~ /OFCL/) && ($line =~ /$synopticDateM6,/)) {
		
			#if ($line =~ /,\sOFCL,\s+0,/) {
			if ($line =~ /,\sOFCL,\s+/) {
			
				push (@{$raOFCL}, $line);
				
			}
			
		}
		
	}
	
	#--- If both the OFCL and CARQ arrays have records then return 1
	
	if (@$raOFCL > 0 && @$raCARQ > 0) {
	
		return 1;	#success
	
	}
	
	return -2;
}



#***********************************************************
#
#	SUBROUTINE NAME: stormDuration
#
#	DESCRIPTION: Opens and reads through a best track file for a 
#				 particular storm and extracts the date of the first
#				 best track record and the date of the last best
#				 track record.  The start and end dates are returned
#				 to the user.
#
#	USAGE (CALLING SEQUENCE):activeStorms()
#
#	VARIABLES AND THEIR USE
#		PASSED: Best Track File name
#				Best Track File directory
#				reference to a variable for the start date
#				reference to a variable for the end date
#				
#		RETURNED: start date in Date::Manip format
#				  end date in Date::Manip format
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub stormDuration {

	#--- Initialize variable
	
	my ($btkFile, $dir, $rStart, $rEnd) = @_;
	my $INPUT = new IO::File;
	
	#--- Read in entire file
	
	my $returnCode = generalUtils::openFile($dir, $btkFile, "read", \$INPUT);
	
	if ($returnCode != 1) {
	
		printf ("Error $returnCode  (atcfStorms::stormDuration): $dir/$btkFile could not be opened for reading \n");
		return $returnCode;
	
	} 
	
	my @lines = <$INPUT>;
	close $INPUT;
	
	if (@lines == 0) {
		print (@lines);
		return -1;	#error empty file
	
	}
	elsif (@lines == 1) {
	
		if (getDTG($lines[0], $rStart) != 1) {
		
			print (@lines);
			return -1;	#error extracting start date
		
		}
		
		$rEnd = $$rStart;	#If there is only one line then the start of the storm
					#equals the end of the storm
	
	}
	elsif ((getDTG($lines[0], $rStart)) && (getDTG($lines[@lines - 1], $rEnd))) {
	
		return 1;
	
	}
	
	print (@lines);
	return -1;

}



#***********************************************************
#
#	SUBROUTINE NAME: getDTG
#
#	DESCRIPTION: Takes a ATCF format string and parses
#				 out the date/time group field.  The DTG
#				 is passed back to the caller in teh $rDate 
#				 variable.
#
#	USAGE (CALLING SEQUENCE): stormDuration()
#
#	VARIABLES AND THEIR USE
#		PASSED: String in ATCF format
#				Reference to a variable to store the date
#
#		RETURNED Date in Date::Manip format
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub getDTG {

	#--- Initialize Variables
	
	my ($line, $rDate) = @_;
	
	#--- Parse the line and extract the date/time group
	
	my @parsedLine = split(/\,/, $line);
 	$$rDate = $parsedLine[2];
	
	#--- Remove white spaces
	
	$$rDate =~ s/\s+//g;
	
	#--- Check date for validity
	
	my $returnCode = generalUtils::validDateFormat($$rDate);
	
	if ($returnCode != 1) {
	
		printf ("Error $returnCode (atcfStorms::getDTG): Date must be a string in YYYYMMDDHH format.\n");
		return -1;
	
	}
	
	#--- Convert string to a date
	
	$$rDate = Date::Manip::ParseDate($$rDate);
	
	return 1;

}



#***********************************************************
#
#	SUBROUTINE NAME: getDate
#
#	DESCRIPTION: Returns the calendar date that the ATCF Object
#				 was initilized with.  This calendar date is 
#				 used to identify a list of active storms and 
#				 then create the computes files.  
#
#	USAGE (CALLING SEQUENCE): external Accessor Method 
#
#	VARIABLES AND THEIR USE
#		PASSED: ATCF Object
#
#		RETURNED: A date as a string in YYYYMMDDHH format
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub getDate {

	#--- Initialize Variables
	
	my ($atcfObj) = @_;

	#--- Return an object in Date::Manip format
	
	return Date::Manip::UnixDate($$atcfObj{date}, '%Y%m%d%H');
	#return $$atcfObj{date};

}



#***********************************************************
#
#	SUBROUTINE NAME: getActiveStorms
#
#	DESCRIPTION: Returns a reference to an array that contains
#				 a list of the active storms.  Active storms are
#				 defined as any tropical cyclones in which best track
#				 records are being written during the date specified
#				 by the user.  The date is specified when the
#				 ATCF object is initialized.
#
#	USAGE (CALLING SEQUENCE): external Accessor Method
#
#	VARIABLES AND THEIR USE
#		PASSED: ATCF Object
#
#		RETURNED: Reference to an array of strings that 
#				  uniquely identify storm names.
#
#.................MAINTENANCE SECTION..................
#                                                                                   
#  LANGUAGE:  Perl 5.8.x                                               
#                                                             
#  RECORD OF CHANGES:                                         
#                                                            
#************************************************************
sub getActiveStorms {

	#--- Initialize Variables
	
	my ($atcfObj) = @_;

	#--- Return a reference to an array of active storms

	return $$atcfObj{storms};

}



1;


# testing file
#
##! /home/akrautkr/perl_5.8.0/bin/perl -w
#
#use strict;
#require "atcfStorms_v4.pl";
#
#my $atcf = atcfStorms::new("/data/atcf", "2003090812");
#
#if (defined $atcf) {
#
#	my $raStorms = $atcf->getActiveStorms();
#	
#	foreach my $storm (@{$raStorms}) {
#	
#		printf ($storm);
#	
#	}
#	
#	my $date = $atcf->getDate();
#	printf ($date);
#	
#	#$atcf->computes();
#	$atcf->stormComputes("al132003");
#	#$atcf->stormComputes("al102003");
#	my $returnCode = $atcf->createStormComputes("al132003", "/home/akrautkr/programs/perl","comab.dat");
#
#	printf ("The return code is $returnCode.\n");
#}
#
#
#
#
